/*global describe,it,expect */
"use strict";

require('../vendor/phantomjs-shim')

var react= require('react')
  , cx = require('../src/util/cx')
  , filters = require('../src/util/filter')
  , transferProps = require('../src/util/transferProps')
  , _ = require('../src/util/_')

describe('when using Class Set', function(){

  it('should concat names', function(){
    expect(cx({ 'a': true, b: true, c: false, d: true})).to.be('a b d')
  })

  it('should ignore empty', function(){
    expect(cx({ 'a': '', b: 0, c: false, d: true})).to.be('d')
  })
  
})


describe('when using prop transfer Utils', function(){

  it('should merge into props object', function(){
    var props = {}
      , newProps = transferProps.mergeIntoProps({a: 'hi'}, props)

    expect(props).to.have.property('a', 'hi')
  })

  it('should merge into descriptor', function(){
    var child = react.createClass({ render: function(){} })({})
      , newProps = transferProps.mergeIntoProps({a: 'hi'}, child)

    expect(child.props).to.have.property('a', 'hi')
  })
  
  it('should merge className', function(){
    var props = { className: 'class hi'}
      , newProps = transferProps.mergeIntoProps({ className: 'new-class' }, props)

    expect(props).to.have.property('className', 'class hi new-class')
  })

  it('should merge style', function(){
    var props = { style: { styleB: true }}
      , newProps = transferProps.mergeIntoProps({ style: { styleA: true } }, props)

    expect(props.style).to.have.keys(['styleA', 'styleB'])
  })

  it('should ignore ref, children ,and key', function(){
    var props = { ref: 'ref', key: 'key', children: [1,2,3] }
      , newProps = transferProps.mergeIntoProps({ ref: 'refB', key: 'keyB', children: [4,5] }, props)

    expect(props).to.have.property('ref', 'ref')
      .and.to.have.property('key', 'key')

    expect(props.children).to.be.eql([1,2,3])
  })
})


describe('_ utils', function(){

  it('should OMIT and PICK', function(){
    expect(_.omit({ a: 1, b: 2, c: 3}, ['b','c'])).to.eql({ a: 1 })
    expect(_.pick({ a: 1, b: 2, c: 3}, ['b','c'])).to.eql({ b: 2, c: 3 })
  })

  it('should FINDINDEX', function(){
    expect(_.findIndex([1,2,3,4,5], v => v === 2)).to.equal(1)
    expect(_.findIndex([1,2,3,4,5], (v, i) => i === 2)).to.equal(2)
  })

  it('should FILTER', function(){
    expect( _.filter([1,2,3,4,5], (v, i) => v > 2 && i < 4 )).to.eql([3,4])
  })

  it('should SOME', function(){
    expect(_.some([1,2,3,4,5], (v, i) => v > 2 )).to.equal(true)
    expect(_.some([1,2,3,4,5], (v, i) => i > 2 )).to.equal(true)
  })

  it('should UNIQUEID', function(){
    expect(_.uniqueId('gello_')).to.equal('gello_' + 1)
    expect(_.uniqueId('ello_')).to.equal('ello_' + 2)
  })

  it('should OBJECT', function(){
    expect(_.object([[1,2], [2,3]])).to.eql({ 1:2, 2: 3})
  })

  it('should SHALLOW EQUAL', function(){
    expect( _.isShallowEqual(1,1) ).to.be(true)
    expect( _.isShallowEqual(1,'1') ).to.be(false)
    expect( _.isShallowEqual(1, 1.4) ).to.be(false)
    expect( _.isShallowEqual('hi', 'hi') ).to.be(true)
    expect( _.isShallowEqual('hi', 'hiw') ).to.be(false)

    expect( _.isShallowEqual(null, null) ).to.be(true)
    expect( _.isShallowEqual(null, undefined) ).to.be(false)

    expect( _.isShallowEqual([1,2], [1,2]) ).to.be(true)
    expect( _.isShallowEqual([1,2], [1,3]) ).to.be(false)

    expect( _.isShallowEqual([1,2], { 0: 1, 1: 2}) ).to.be(true)

    expect( _.isShallowEqual({ a: 1, b: 2}, { b: 2, a: 1 }) ).to.be(true)
    expect( _.isShallowEqual({ a: 1, b: 2}, { a: 1, c: 'hi' }) ).to.be(false)
  })

  it('should TRANSFORM', function(){
    var obj;

    _.transform([1], function(o, v,i){ 
      expect(o).to.eql([])
      expect(v).to.equal(1)
      expect(i).to.equal(0)
    })

    _.transform({ key: 1 }, (o, v, i) => { 
      expect(o).to.eql({})
      expect(v).to.equal(1)
      expect(i).to.equal('key')
    })

    expect( _.transform({ a:0, b:1 }, 
      (o, v, i) => o[i] = ++v )).to.eql({ a:1, b: 2})

    expect( _.transform([ 0, 1 ], 
      (o, v ) => o[v] = ++v, {})).to.eql({ 0:1, 1: 2})
  })
})


describe('when using array filter helpers', function(){

  it('should match correctly', function(){

    expect(filters.eq(1, 1)).to.equal(true)
    expect(filters.neq(2, 1)).to.equal(true)
    expect(filters.lt(1, 2)).to.equal(true)
    expect(filters.lte(1, 1)).to.equal(true)
    expect(filters.gt(2, 1)).to.equal(true)
    expect(filters.gte(1, 1)).to.equal(true)
   
    expect(filters.contains([1,2], 1)).to.equal(true)
    expect(filters.contains('hello', 'll')).to.equal(true)

    expect(filters.startsWith('hello', 'hel')).to.equal(true)
    expect(filters.endsWith('hello', 'llo')).to.equal(true)
  })
})

describe('when using date helpers', function(){

  it('should match correctly', function(){

    expect(filters.eq(1, 1)).to.equal(true)
    expect(filters.neq(2, 1)).to.equal(true)
    expect(filters.lt(1, 2)).to.equal(true)
    expect(filters.lte(1, 1)).to.equal(true)
    expect(filters.gt(2, 1)).to.equal(true)
    expect(filters.gte(1, 1)).to.equal(true)
   
    expect(filters.contains([1,2], 1)).to.equal(true)
    expect(filters.contains('hello', 'll')).to.equal(true)

    expect(filters.startsWith('hello', 'hel')).to.equal(true)
    expect(filters.endsWith('hello', 'llo')).to.equal(true)
  })
})